@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.32 2008/05/20 18:38:05 msk Exp $";
+static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.40 2008/07/22 22:07:23 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -22,6 +22,9 @@ static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.32 2008/05/20 18:38
 #include <ctype.h>
 
 /* libsm includes */
+#include <sm/gen.h>
+#include <sm/types.h>
+#include <sm/cdefs.h>
 #include <sm/string.h>
 
 /* libar includes */
@@ -50,6 +53,10 @@ extern void dkim_error __P((DKIM *, const char *, ...));
 # define RES_UNC_T		unsigned char *
 #endif /* __RES && __RES >= 19940415 */
 
+#ifndef T_AAAA
+# define T_AAAA	28
+#endif /* ! T_AAAA */
+
 /*
 **  DKIM_GET_POLICY_FILE -- acquire a domain's policy record using a local file
 **
@@ -144,13 +151,305 @@ dkim_get_policy_file(DKIM *dkim, unsigned char *query, unsigned char *buf,
 	return 0;
 }
 
+/*
+**  DKIM_GET_POLICY_DNS_EXCHECK -- existence check for a name
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	query -- query to execute
+**  	qstatus -- query result code (DNS-style)
+**
+**  Return value:
+**  	1 -- domain exists
+**  	0 -- domain does not exist
+**  	-1 failure
+*/
+
+int
+dkim_get_policy_dns_excheck(DKIM *dkim, unsigned char *query, int *qstatus)
+{
+	int c;
+	size_t anslen_a;
+	size_t anslen_aaaa;
+	size_t anslen_mx;
+	int status;
+	DKIM_LIB *lib;
+	HEADER hdr;
+#if USE_ARLIB
+	AR_LIB ar;
+	AR_QUERY q_a;
+	AR_QUERY q_aaaa;
+	AR_QUERY q_mx;
+	int arerror_a;
+	int arerror_aaaa;
+	int arerror_mx;
+#endif /* USE_ARLIB */
+	struct timeval timeout;
+	unsigned char ansbuf_a[MAXPACKET];
+	unsigned char ansbuf_aaaa[MAXPACKET];
+	unsigned char ansbuf_mx[MAXPACKET];
+
+	assert(dkim != NULL);
+	assert(query != NULL);
+	assert(qstatus != NULL);
+
+	lib = dkim->dkim_libhandle;
+
+#if USE_ARLIB
+# ifdef _FFR_DNS_UPGRADE
+	for (c = 0; c < 2; c++)
+	{
+		switch (c)
+		{
+		  case 0:
+			ar = dkim->dkim_libhandle->dkiml_arlib;
+			break;
+
+		  case 1:
+			ar = dkim->dkim_libhandle->dkiml_arlibtcp;
+			break;
+		}
+
+		timeout.tv_sec = dkim->dkim_timeout;
+		timeout.tv_usec = 0;
+
+		q_a = ar_addquery(ar, query, C_IN, T_A, MAXCNAMEDEPTH,
+		                  ansbuf_a, sizeof ansbuf_a, &arerror_a,
+		                  dkim->dkim_timeout == 0 ? NULL
+		                                          : &timeout);
+		if (q_a == NULL)
+		{
+			dkim_error(dkim, "ar_addquery() for `%s' failed",
+			           query);
+			return -1;
+		}
+
+		q_aaaa = ar_addquery(ar, query, C_IN, T_AAAA, MAXCNAMEDEPTH,
+		                     ansbuf_aaaa, sizeof ansbuf_aaaa,
+		                     &arerror_aaaa,
+		                     dkim->dkim_timeout == 0 ? NULL
+		                                             : &timeout);
+		if (q_aaaa == NULL)
+		{
+			dkim_error(dkim, "ar_addquery() for `%s' failed",
+			           query);
+			return -1;
+		}
+
+		q_mx = ar_addquery(ar, query, C_IN, T_MX, MAXCNAMEDEPTH,
+		                   ansbuf_mx, sizeof ansbuf_mx,
+		                   &arerror_mx,
+		                   dkim->dkim_timeout == 0 ? NULL
+		                                           : &timeout);
+		if (q_mx == NULL)
+		{
+			dkim_error(dkim, "ar_addquery() for `%s' failed",
+			           query);
+			return -1;
+		}
+
+		if (lib->dkiml_dns_callback == NULL)
+		{
+			status = ar_waitreply(ar, q_a, &anslen_a, NULL);
+			status = ar_waitreply(ar, q_aaaa, &anslen_aaaa, NULL);
+			status = ar_waitreply(ar, q_mx, &anslen_mx, NULL);
+		}
+		else
+		{
+			int which = 0;
+ 
+			while (which <= 2)
+			{
+				timeout.tv_sec = lib->dkiml_callback_int;
+				timeout.tv_usec = 0;
+
+				switch (which)
+				{
+				  case 0:
+					status = ar_waitreply(ar, q_a,
+					                      &anslen_a,
+					                      &timeout);
+					break;
+
+				  case 1:
+					status = ar_waitreply(ar, q_aaaa,
+					                      &anslen_aaaa,
+					                      &timeout);
+					break;
+
+				  case 2:
+					status = ar_waitreply(ar, q_mx,
+					                      &anslen_mx,
+					                      &timeout);
+					break;
+				}
+
+				if (status != AR_STAT_NOREPLY)
+				{
+					if (which == 2)
+					{
+						break;
+					}
+					else
+					{
+						which++;
+						continue;
+					}
+				}
+
+				lib->dkiml_dns_callback(dkim->dkim_user_context);
+			}
+		}
+
+		(void) ar_cancelquery(ar, q_a);
+		(void) ar_cancelquery(ar, q_aaaa);
+		(void) ar_cancelquery(ar, q_mx);
+
+		/* see if any of the UDP replies was truncated */
+		if (c == 0 && status == AR_STAT_SUCCESS)
+		{
+			if (dkim_check_dns_reply(ansbuf_a, anslen_a,
+			                         C_IN, T_TXT) == 1)
+				continue;
+
+			if (dkim_check_dns_reply(ansbuf_aaaa, anslen_aaaa,
+			                         C_IN, T_TXT) == 1)
+				continue;
+
+			if (dkim_check_dns_reply(ansbuf_mx, anslen_mx,
+			                         C_IN, T_TXT) == 1)
+				continue;
+		}
+
+		break;
+	}
+# else /* _FFR_DNS_UPGRADE */
+	ar = dkim->dkim_libhandle->dkiml_arlib;
+
+	timeout.tv_sec = dkim->dkim_timeout;
+	timeout.tv_usec = 0;
+
+	q_a = ar_addquery(ar, query, C_IN, T_A, MAXCNAMEDEPTH, ansbuf_a,
+	                  sizeof ansbuf_a, &arerror_a,
+	                  dkim->dkim_timeout == 0 ? NULL : &timeout);
+	if (q_a == NULL)
+	{
+		dkim_error(dkim, "ar_addquery() failed for `%s'",
+		           query);
+		return -1;
+	}
+
+	q_aaaa = ar_addquery(ar, query, C_IN, T_AAAA, MAXCNAMEDEPTH,
+	                     ansbuf_aaaa, sizeof ansbuf_aaaa, &arerror_aaaa,
+	                     dkim->dkim_timeout == 0 ? NULL : &timeout);
+	if (q_aaaa == NULL)
+	{
+		dkim_error(dkim, "ar_addquery() failed for `%s'",
+		           query);
+		return -1;
+	}
+
+	q_mx = ar_addquery(ar, query, C_IN, T_MX, MAXCNAMEDEPTH,
+	                   ansbuf_mx, sizeof ansbuf_mx, &arerror_mx,
+	                   dkim->dkim_timeout == 0 ? NULL : &timeout);
+	if (q_mx == NULL)
+	{
+		dkim_error(dkim, "ar_addquery() failed for `%s'",
+		           query);
+		return -1;
+	}
+
+	if (lib->dkiml_dns_callback == NULL)
+	{
+		status = ar_waitreply(ar, q_a, &anslen_a, NULL);
+		status = ar_waitreply(ar, q_aaaa, &anslen_aaaa, NULL);
+		status = ar_waitreply(ar, q_mx, &anslen_mx, NULL);
+	}
+	else
+	{
+		int which = 0;
+ 
+		while (which <= 2)
+		{
+			timeout.tv_sec = lib->dkiml_callback_int;
+			timeout.tv_usec = 0;
+
+			switch (which)
+			{
+			  case 0:
+				status = ar_waitreply(ar, q_a,
+				                      &anslen_a,
+				                      &timeout);
+				break;
+
+			  case 1:
+				status = ar_waitreply(ar, q_aaaa,
+				                      &anslen_aaaa,
+				                      &timeout);
+				break;
+
+			  case 2:
+				status = ar_waitreply(ar, q_mx,
+				                      &anslen_mx,
+				                      &timeout);
+				break;
+			}
+
+			if (status != AR_STAT_NOREPLY)
+			{
+				if (which == 2)
+				{
+					break;
+				}
+				else
+				{
+					which++;
+					continue;
+				}
+			}
+
+			lib->dkiml_dns_callback(dkim->dkim_user_context);
+		}
+	}
+
+	(void) ar_cancelquery(ar, q_a);
+	(void) ar_cancelquery(ar, q_aaaa);
+	(void) ar_cancelquery(ar, q_mx);
+# endif /* _FFR_DNS_UPGRADE */
+#else /* USE_ARLIB */
+	status = res_query(query, C_IN, T_A, ansbuf_a, sizeof ansbuf_a);
+	status = res_query(query, C_IN, T_AAAA, ansbuf_aaaa,
+	                   sizeof ansbuf_aaaa);
+	status = res_query(query, C_IN, T_MX, ansbuf_mx, sizeof ansbuf_mx);
+#endif /* USE_ARLIB */
+
+	/* check each for NXDOMAIN or some other issue */
+	memcpy(&hdr, ansbuf_a, sizeof hdr);
+	*qstatus = hdr.rcode;
+	if (hdr.rcode == NOERROR)
+		return 1;
+
+	memcpy(&hdr, ansbuf_aaaa, sizeof hdr);
+	*qstatus = hdr.rcode;
+	if (hdr.rcode == NOERROR)
+		return 1;
+
+	memcpy(&hdr, ansbuf_mx, sizeof hdr);
+	*qstatus = hdr.rcode;
+	if (hdr.rcode == NOERROR)
+		return 1;
+
+	/* looks good */
+	return 0;
+}
+
 /*
 **  DKIM_GET_POLICY_DNS -- acquire a domain's policy record using DNS queries
 **
 **  Parameters:
 **  	dkim -- DKIM handle
 **  	query -- query to execute
-**  	usemx -- use an MX record?
+**  	excheck -- existence check?
 **  	buf -- buffer into which to write policy
 **  	buflen -- number of bytes available at "buf"
 **  	qstatus -- query result code (DNS-style)
@@ -162,7 +461,7 @@ dkim_get_policy_file(DKIM *dkim, unsigned char *query, unsigned char *buf,
 */
 
 int
-dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
+dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool excheck,
                     unsigned char *buf, size_t buflen, int *qstatus)
 {
 	int qdcount;
@@ -170,7 +469,6 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 	int status;
 	int n;
 	int c;
-	int qtype;
 	int type = -1;
 	int class = -1;
 #ifdef QUERY_CACHE
@@ -221,12 +519,16 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 	}
 #endif /* QUERY_CACHE */
 
-	qtype = (usemx ? T_MX : T_TXT);
-
 	/* see if there's a simulated reply queued; if so, use it */
 	anslen = dkim_test_dns_get(dkim, ansbuf, sizeof ansbuf);
 	if (anslen == -1)
 	{
+		if (excheck)
+		{
+			return dkim_get_policy_dns_excheck(dkim, query,
+			                                   qstatus);
+		}
+
 #if USE_ARLIB
 # ifdef _FFR_DNS_UPGRADE
 		for (c = 0; c < 2; c++)
@@ -245,7 +547,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 			timeout.tv_sec = dkim->dkim_timeout;
 			timeout.tv_usec = 0;
 
-			q = ar_addquery(ar, query, C_IN, qtype, MAXCNAMEDEPTH,
+			q = ar_addquery(ar, query, C_IN, T_TXT, MAXCNAMEDEPTH,
 			                ansbuf, sizeof ansbuf, &arerror,
 			                dkim->dkim_timeout == 0 ? NULL
 			                                        : &timeout);
@@ -297,7 +599,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 		timeout.tv_sec = dkim->dkim_timeout;
 		timeout.tv_usec = 0;
 
-		q = ar_addquery(ar, query, C_IN, qtype, MAXCNAMEDEPTH, ansbuf,
+		q = ar_addquery(ar, query, C_IN, T_TXT, MAXCNAMEDEPTH, ansbuf,
 		                sizeof ansbuf, &arerror,
 		                dkim->dkim_timeout == 0 ? NULL : &timeout);
 		if (q == NULL)
@@ -332,7 +634,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 # endif /* _FFR_DNS_UPGRADE */
 #else /* USE_ARLIB */
 
-		status = res_query(query, C_IN, qtype, ansbuf, sizeof ansbuf);
+		status = res_query(query, C_IN, T_TXT, ansbuf, sizeof ansbuf);
 
 #endif /* USE_ARLIB */
 
@@ -421,7 +723,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 		GETSHORT(class, cp);
 	}
 
-	if (type != qtype || class != C_IN)
+	if (type != T_TXT || class != C_IN)
 	{
 		dkim_error(dkim, "`%s' unexpected reply class/type", query);
 		return -1;
@@ -476,7 +778,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 			cp += n;
 			continue;
 		}
-		else if (type != qtype)
+		else if (type != T_TXT)
 		{
 			/* reject anything not valid (e.g. wildcards) */
 			dkim_error(dkim, "`%s' unexpected reply class/type",
@@ -484,7 +786,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 			return -1;
 		}
 
-		if (!usemx && ancount > 0)
+		if (ancount > 0)
 		{
 			dkim_error(dkim, "multiple DNS replies for `%s'",
 			           query);
@@ -501,15 +803,6 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 		return -1;
 	}
 
-	/*
-	**  If we're querying something other than a T_TXT here, it was
-	**  used by part of the policy algorithm merely to validate the
-	**  existence of a domain; the payload isn't really required.
-	*/
-
-	if (usemx)
-		return 1;
-
 #ifdef QUERY_CACHE
 	GETLONG(ttl, cp);
 #else /* QUERY_CACHE */
@@ -535,11 +828,12 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 	/* extract the payload */
 	memset(outbuf, '\0', sizeof outbuf);
 	p = outbuf;
-	while (n > 0)
+	eom = outbuf + sizeof outbuf - 1;
+	while (n > 0 && p < eom)
 	{
 		c = *cp++;
 		n--;
-		while (c > 0)
+		while (c > 0 && p < eom)
 		{
 			*p++ = *cp++;
 			c--;
