@@ -7,18 +7,13 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.182 2008/06/05 15:19:36 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.184 2008/07/22 20:42:38 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
 #include <sys/types.h>
 #include <sys/param.h>
 
-/* libsm includes */
-#include <sm/gen.h>
-#include <sm/types.h>
-#include <sm/cdefs.h>
-
 /* openssl includes */
 #include <openssl/sha.h>
 
@@ -31,7 +26,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.182 2008/06/05 15:19:36 msk Exp $
 **  	pp == patch number
 */
 
-#define	DKIM_LIB_VERSION	0x02060000
+#define	DKIM_LIB_VERSION	0x02070000
 
 /* the basics */
 #ifndef NULL
@@ -99,7 +94,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.182 2008/06/05 15:19:36 msk Exp $
 /* special DNS tokens */
 #define	DKIM_DNSKEYNAME		"_domainkey"
 					/* reserved DNS sub-zone */
-#define	DKIM_DNSPOLICYNAME	"_asp"	/* reserved DNS sub-zone */
+#define	DKIM_DNSPOLICYNAME	"_adsp"	/* reserved DNS sub-zone */
 
 /*
 **  DKIM_STAT -- status code type
@@ -305,10 +300,7 @@ typedef int dkim_policy_t;
 
 #define DKIM_PRESULT_NONE		(-1)	/* none/undefined */
 #define DKIM_PRESULT_NXDOMAIN		0	/* domain does not exist */
-#define DKIM_PRESULT_AUTHOR		1	/* ASP found for author */
-#define DKIM_PRESULT_NOPARENT		2	/* no parent ASP found */
-#define DKIM_PRESULT_NOSUBDOMAIN	3	/* parent ASP disallows subs */
-#define DKIM_PRESULT_PARENT		4	/* ASP found for parent */
+#define DKIM_PRESULT_AUTHOR		1	/* ADSP found for author */
 
 /*
 **  DKIM_MODE -- mode of a handle
@@ -1100,7 +1092,7 @@ extern DKIM_STAT dkim_set_key_lookup __P((DKIM_LIB *libdkim,
 extern DKIM_STAT dkim_set_policy_lookup __P((DKIM_LIB *libdkim,
                                              DKIM_CBSTAT (*func)(DKIM *dkim,
                                                                  u_char *query,
-                                                                 bool usemx,
+                                                                 bool excheck,
                                                                  u_char *buf,
                                                                  size_t buflen,
                                                                  int *qstat)));
