@@ -4,13 +4,16 @@
 */
 
 #ifndef lint
-static char t_cleanup_c_id[] = "@(#)$Id: t-cleanup.c,v 1.2 2007/05/02 02:38:19 msk Exp $";
+static char t_cleanup_c_id[] = "@(#)$Id: t-cleanup.c,v 1.3 2008/07/22 20:51:17 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
 #include <assert.h>
 #include <unistd.h>
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* libdkim includes */
 #include "t-testdata.h"
 
