@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test48_c_id[] = "@(#)$Id: t-test48.c,v 1.6 2007/07/27 05:39:47 msk Exp $";
+static char t_test48_c_id[] = "@(#)$Id: t-test48.c,v 1.7 2008/07/22 20:51:17 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -13,6 +13,9 @@ static char t_test48_c_id[] = "@(#)$Id: t-test48.c,v 1.6 2007/07/27 05:39:47 msk
 #include <string.h>
 #include <stdio.h>
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* libdkim includes */
 #include "dkim.h"
 #include "t-testdata.h"
