@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test54_c_id[] = "@(#)$Id: t-test54.c,v 1.8 2008/06/05 15:23:18 msk Exp $";
+static char t_test54_c_id[] = "@(#)$Id: t-test54.c,v 1.9 2008/07/22 20:51:17 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -13,6 +13,9 @@ static char t_test54_c_id[] = "@(#)$Id: t-test54.c,v 1.8 2008/06/05 15:23:18 msk
 #include <string.h>
 #include <stdio.h>
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* libdkim includes */
 #include "dkim.h"
 #include "t-testdata.h"
