@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test65_c_id[] = "@(#)$Id: t-test65.c,v 1.5 2008/06/05 15:23:18 msk Exp $";
+static char t_test65_c_id[] = "@(#)$Id: t-test65.c,v 1.6 2008/07/22 20:51:17 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -14,6 +14,9 @@ static char t_test65_c_id[] = "@(#)$Id: t-test65.c,v 1.5 2008/06/05 15:23:18 msk
 #include <stdio.h>
 #include <stdlib.h>
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* libdkim includes */
 #include "dkim.h"
 #include "t-testdata.h"
