@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test69_c_id[] = "@(#)$Id: t-test69.c,v 1.3 2007/09/17 16:36:45 msk Exp $";
+static char t_test69_c_id[] = "@(#)$Id: t-test69.c,v 1.4 2008/07/22 20:51:17 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -13,6 +13,9 @@ static char t_test69_c_id[] = "@(#)$Id: t-test69.c,v 1.3 2007/09/17 16:36:45 msk
 #include <string.h>
 #include <stdio.h>
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* libdkim includes */
 #include "dkim.h"
 #include "t-testdata.h"
