@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test70_c_id[] = "@(#)$Id: t-test70.c,v 1.3 2007/12/15 01:29:24 msk Exp $";
+static char t_test70_c_id[] = "@(#)$Id: t-test70.c,v 1.4 2008/07/22 20:51:17 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -14,6 +14,9 @@ static char t_test70_c_id[] = "@(#)$Id: t-test70.c,v 1.3 2007/12/15 01:29:24 msk
 #include <stdio.h>
 #include <regex.h>		/* for TRE_APPROX */
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* libdkim includes */
 #include "dkim.h"
 #include "t-testdata.h"
